searchData={"items":[{"type":"behaviour","title":"Vela","doc":"`Vela` is a tiny library providing easy management of\nvalidated cached state with some history.\n\nIncluding `use Vela` in your module would turn the module\ninto struct, setting field accordingly to the specification,\npassed as a parameter.\n\n`Vela` allows the following configurable parameters per field:\n\n- `limit` — length of the series to keep (default: `5`)\n- `compare_by` — comparator extraction function to extract the value, to be used for\n  comparison, from the underlying terms (default: `& &1`)\n- `comparator` — the function that accepts a series name and two values and returns\n  the greater one to be used in `Vela.δ/1` (default: `&</2`)\n- `threshold` — if specified, the inserted value is checked to fit in `δ ± threshold`;\n  whether it does not, it goes to `errors` (`float() | nil`, default: `nil`)\n- `validator` — the function to be used to invalidate the accumulated values (default:\n  `fn _ -> true end`)\n- `sorter` — the function to be used to sort values within one serie, if\n  none is given, it sorts in the natural order, FIFO, newest is the one to `pop`\n- `corrector` — the function to be used to correct the values rejected by `validator`;\n  the function should return `{:ok, corrected_value}` to enforce insertion into `Vela`,\n  or `:error` if the value cannot be corrected and should be nevertheless rejected\n- `errors` — number of errors to keep (default: `5`)\n\nAlso, Vela accepts `:__meta__` keyword parameter for the cases when the consumer needs\nthe very custom meta to be passed to the struct.\n\n`Vela` implements `Access` behaviour.","ref":"Vela.html"},{"type":"behaviour","title":"Usage - Vela","doc":"defmodule Vela.Test do\n      use Vela,\n        series1: [limit: 3, errors: 1], # no validation\n        series2: [limit: 2, validator: Vela.Test]\n        series3: [\n              compare_by: &Vela.Test.comparator/1,\n              validator: &Vela.Test.validator/2\n        ]\n\n      @behaviour Vela.Validator\n\n      @impl Vela.Validator\n      def valid?(_serie, value), do: value > 0\n\n      @spec comparator(%{created_at :: DateTime.t()}) :: DateTime.t()\n      def comparator(%{created_at: created_at}),\n        do: created_at\n\n      @spec validator(value :: t()) :: boolean()\n      def validator(value),\n        do: is_integer(value) and value > 300\n    end\n\nIn the example above, before any structure update attempt\n(via `Access`,) this `valid?/2` function would be called.\n\nIf it returns `true`, the value gets inserted / updated, and\nthe series behind is truncated if needed. It it returns `false`,\nthe state _is not updated_, and the value is put into the map\nunder `__errors__` key of the struct. The length of errors\nis also configurable via `errors:` keyword parameter.","ref":"Vela.html#module-usage"},{"type":"callback","title":"Vela.average/2","doc":"Returns a keyword with series as keys and the average value as a value\n\n_Example:_\n\n    iex> defmodule XY do\n    ...>   use Vela, x: [], y: []\n    ...> end\n    ...> XY.average(struct(XY, x: [1, 2, 3], y: [5, 7, 9]), &(Enum.sum(&1) / length(&1)))\n    [x: 2.0, y: 7.0]\n    ...> defmodule Averager do\n    ...>   def average(values), do: Enum.sum(values) / length(values)\n    ...> end\n    ...> XY.average(struct(XY, x: [1, 2, 3], y: [5, 7, 9]), Averager)\n    [x: 2.0, y: 7.0]","ref":"Vela.html#c:average/2"},{"type":"callback","title":"Vela.config/3","doc":"Returns configuration needed to proceed with the interface `Vela` exposes.\n\n_Example:_\n\n    iex> defmodule VC do\n    ...>   use Vela, a: [validator: VC], b: [], c: []\n    ...>   @behaviour Vela.Validator\n    ...>   @impl Vela.Validator\n    ...>   def valid?(:a, value), do: not is_nil(value)\n    ...> end\n    ...> VC.config(:a, :comparator, struct(VC, []))","ref":"Vela.html#c:config/3"},{"type":"callback","title":"Vela.delta/2","doc":"Returns `{min, max}` tuple for each serie, using the comparator given as a second parameter,\n  or a default comparator for this serie.","ref":"Vela.html#c:delta/2"},{"type":"function","title":"Vela.empty!/1","doc":"Empties the values for all series of `Vela` given.","ref":"Vela.html#empty!/1"},{"type":"callback","title":"Vela.empty!/1","doc":"Empties series in `Vela` given as an input, meta stays untouched.","ref":"Vela.html#c:empty!/1"},{"type":"function","title":"Vela.empty?/1","doc":"Returns `true` if the `Vela` is empty, `false` otherwise.","ref":"Vela.html#empty?/1"},{"type":"callback","title":"Vela.empty?/1","doc":"Checks if `Vela` given as an input is empty","ref":"Vela.html#c:empty?/1"},{"type":"function","title":"Vela.equal?/2","doc":"Returns `true` if velas given as arguments are of the same type _and_\nseries values equal for each serie.\n\nThis function does not check internal state, only the values.","ref":"Vela.html#equal?/2"},{"type":"callback","title":"Vela.equal?/2","doc":"Checks two velas given as an input for equality","ref":"Vela.html#c:equal?/2"},{"type":"function","title":"Vela.flat_map/2","doc":"Flat maps the series using `fun` and returns the keyword with\nduplicated keys and mapped values.\n\n_Example:_\n\n```elixir\ndefmodule EO do\n  use Vela,\n    even: [limit: 2],\n    odd: [limit: 2]\n\n  def flat_map(%EO{} = v),\n    do: Vela.flat_map(v, & {&1, &2+1})\nend\nEO.flat_map(struct(EO, [even: [2, 4], odd: [1, 3]]))\n\n#⇒ [even: 3, even: 5, odd: 2, odd: 4]\n```","ref":"Vela.html#flat_map/2"},{"type":"function","title":"Vela.map/2","doc":"Maps the series using `fun` and returns the new `Vela` instance with series mapped","ref":"Vela.html#map/2"},{"type":"function","title":"Vela.merge/3","doc":"Merges two `Vela`s given using `resolver/3` function.\n\nThis function does not allow merging states, the first argument wins. To update state,\nuse `update_state/2`.","ref":"Vela.html#merge/3"},{"type":"callback","title":"Vela.merge/3","doc":"Merges two `Vela`s, using `resolver/3` given as the third argument in a case of ambiguity.\n\n_Used by:_ `Vela.merge/3`.","ref":"Vela.html#c:merge/3"},{"type":"callback","title":"Vela.purge/2","doc":"Removes obsoleted elements from the series using the validator given as a second parameter,\n  or a default validator for this serie.","ref":"Vela.html#c:purge/2"},{"type":"function","title":"Vela.put/3","doc":"Inserts the new value into the serie, going through all the validation and sorting.\n\nIf the value has not passed validation, it’s put into `:__errors__` internal list.\nIf the new length of the serie exceeds the limit set for this serie, the last value\n(after sorting) gets discarded.","ref":"Vela.html#put/3"},{"type":"callback","title":"Vela.series/0","doc":"Returns series defined by this `Vela` as a list of atoms.\n\n_Example:_\n\n    iex> defmodule V do\n    ...>   use Vela, a: [], b: [], c: []\n    ...> end\n    ...> V.series()\n    [:a, :b, :c]","ref":"Vela.html#c:series/0"},{"type":"callback","title":"Vela.slice/1","doc":"Returns a keyword with series as keys and the hottest value as a value\n\n_Example:_\n\n    iex> defmodule AB do\n    ...>   use Vela, a: [], b: [], c: []\n    ...> end\n    ...> AB.slice(struct(AB, [a: [1, 2], b: [3], c: []]))\n    [a: 1, b: 3]","ref":"Vela.html#c:slice/1"},{"type":"function","title":"Vela.slice/2","doc":"Slices the `Vela` given as a first agrument, returning the keyword with series and _topmost_\nvalue as a value.\n\nThe second argument might be:\n\n- `{:average, averager}` to return an averaged value, assuming `c:Vela.average/2` callback is defined,\n- `:slice` to return the sliced value in the list of values, assuming `c:Vela.slice/1` is defined,\n- `({serie, values} -> value)` function to perform slicing ad-hoc","ref":"Vela.html#slice/2"},{"type":"function","title":"Vela.δ/2","doc":"Returns `{min, max}` tuple for each serie, using the comparator given as a second parameter,\n  or a default comparator for each serie.","ref":"Vela.html#%CE%B4/2"},{"type":"type","title":"Vela.comparator/0","doc":"The type of comparator function to be passed as `:comparator` keyword parameter\n  to the series.","ref":"Vela.html#t:comparator/0"},{"type":"type","title":"Vela.corrector/0","doc":"The type of sorter function to be passed as `:sorter` keyword parameter\n  to the series.","ref":"Vela.html#t:corrector/0"},{"type":"type","title":"Vela.exposed_option/0","doc":"Configuration options used by the outmost world","ref":"Vela.html#t:exposed_option/0"},{"type":"type","title":"Vela.kv/0","doc":"Represents a key-value pair in errors and unmatched","ref":"Vela.html#t:kv/0"},{"type":"type","title":"Vela.option/0","doc":"Options allowed in series configuration","ref":"Vela.html#t:option/0"},{"type":"type","title":"Vela.options/0","doc":"Series configuration","ref":"Vela.html#t:options/0"},{"type":"type","title":"Vela.serie/0","doc":"Represents a key in the Vela structure","ref":"Vela.html#t:serie/0"},{"type":"type","title":"Vela.sorter/0","doc":"The type of sorter function to be passed as `:sorter` keyword parameter\n  to the series.","ref":"Vela.html#t:sorter/0"},{"type":"type","title":"Vela.state/0","doc":"Represents the internal state aka per-vela property container","ref":"Vela.html#t:state/0"},{"type":"type","title":"Vela.t/0","doc":"Represents the struct created by this behaviour module","ref":"Vela.html#t:t/0"},{"type":"type","title":"Vela.validator/0","doc":"The type of validator function to be passed as `:validator` keyword parameter\n  to the series.","ref":"Vela.html#t:validator/0"},{"type":"type","title":"Vela.value/0","doc":"Represents a value in the Vela structure","ref":"Vela.html#t:value/0"},{"type":"behaviour","title":"Vela.Validator","doc":"The validator behaviour, implementations are used by `Vela`\nto validate new values to be inserted into the state.","ref":"Vela.Validator.html"},{"type":"callback","title":"Vela.Validator.valid?/2","doc":"When implemented, the module might be passed as validator to `use Vela`","ref":"Vela.Validator.html#c:valid?/2"},{"type":"module","title":"WeatherForecast","doc":"The example of `Vela` implementation, demontrating the real-life\nmodule using `Vela` for timeseries control. \n\nThe full code of this file follows.\n\n```elixir\ndefmodule WeatherForecast do\n  alias WeatherForecast, as: Me\n  use Vela,\n    __globals__: [validator: Me],\n    tomorrow: [limit: 5, errors: 1],\n    two_days: [limit: 3],\n    week: [limit: 2, sorter: &Me.sort/2]\n\n  @behaviour Vela.Validator\n\n  @impl Vela.Validator\n  def valid?(_serie, value), do: value > 0\n\n  @doc \"Sort function to be used as `:sorter` in series\"\n  def sort(v1, v2), do: v1 <= v2\nend\n```","ref":"WeatherForecast.html"},{"type":"function","title":"WeatherForecast.average/2","doc":"Implementation of `c:Vela.average/2`.\n\nReturns a slice of all series as `keyword()` in format `{Vela.serie(), Vela.value()}`,\nwhen the `value` is a calculated average of all serie values.\n\nThe second parameter might be either a function of arity `1`, accepting a serie (a list of values),\nor a module, exporting `average/1` function.\n\nThe result is similar to what `slice/1` returns, but with average values.","ref":"WeatherForecast.html#average/2"},{"type":"function","title":"WeatherForecast.delta/2","doc":"Implementation of `c:Vela.delta/2`.\n\nDelegates to `Vela.δ/2`.","ref":"WeatherForecast.html#delta/2"},{"type":"function","title":"WeatherForecast.empty!/1","doc":"Empties the `Vela` given as an argument, preserving all the internal information\n(`__meta__`, `__errors__` etc.)\n\n_See:_ `Vela.empty!/1`.","ref":"WeatherForecast.html#empty!/1"},{"type":"function","title":"WeatherForecast.empty?/1","doc":"Returns `true` if there are no values in all series, `false` otherwise.\nMore performant implementation of `slice(vela) == []`.\n\n_See:_ `Vela.empty?/1`.","ref":"WeatherForecast.html#empty?/1"},{"type":"function","title":"WeatherForecast.equal?/2","doc":"Implementation of `c:Vela.equal?/2`.\n\nDelegates to `Vela.equal?/2`.","ref":"WeatherForecast.html#equal?/2"},{"type":"function","title":"WeatherForecast.merge/3","doc":"Merges two `Vela`s, using `resolver/3` given as the third argument in a case of ambiguity.\n\n_See:_ `Vela.merge/3`.","ref":"WeatherForecast.html#merge/3"},{"type":"function","title":"WeatherForecast.purge/2","doc":"Implementation of `c:Vela.purge/2`.\n\nPurges values which are not passing `validator` given as a second parameter.","ref":"WeatherForecast.html#purge/2"},{"type":"function","title":"WeatherForecast.series/0","doc":"Returns the list of series declared on `WeatherForecast`","ref":"WeatherForecast.html#series/0"},{"type":"function","title":"WeatherForecast.slice/1","doc":"Implementation of `c:Vela.slice/1`.\n\nReturns a slice of all series as `keyword()` in format `{Vela.serie(), Vela.value()}`,\nif the series has no values, it’s not included into result.","ref":"WeatherForecast.html#slice/1"},{"type":"function","title":"WeatherForecast.sort/2","doc":"Sort function to be used as `:sorter` in series","ref":"WeatherForecast.html#sort/2"},{"type":"function","title":"WeatherForecast.state/1","doc":"Returns the initial state of `WeatherForecast`, custom options etc","ref":"WeatherForecast.html#state/1"},{"type":"function","title":"WeatherForecast.update_state/2","doc":"Updates the internal state of `WeatherForecast`","ref":"WeatherForecast.html#update_state/2"},{"type":"function","title":"WeatherForecast.vela_config/0","doc":"Returns the config `WeatherForecast` was declared with","ref":"WeatherForecast.html#vela_config/0"},{"type":"type","title":"WeatherForecast.serie_slice/0","doc":"The possibly type of members of the result of call to `Vela.slice/1`","ref":"WeatherForecast.html#t:serie_slice/0"},{"type":"type","title":"WeatherForecast.slice/0","doc":"The result of call to `Vela.slice/1`","ref":"WeatherForecast.html#t:slice/0"},{"type":"type","title":"WeatherForecast.t/0","doc":"The type of this particular `Vela` implementation","ref":"WeatherForecast.html#t:t/0"},{"type":"exception","title":"Vela.AccessError","doc":"The generic exception thrown when the access in invalid.\n\nParameters:\n\n- `source` the source of the exception\n- `field` the field the access to has failed","ref":"Vela.AccessError.html"},{"type":"extras","title":"Vela","doc":"# ![Vela](https://raw.githubusercontent.com/am-kantox/vela/master/stuff/vela-48x48.png) Vela    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/vela/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/vela/workflows/Dialyzer/badge.svg)\n\n**The tiny library to ease handling expiring invalidated cached series**","ref":"readme.html"},{"type":"extras","title":"Installation - Vela","doc":"```elixir\ndef deps do\n  [\n    {:vela, \"~> 0.12\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Changelog - Vela","doc":"- **`1.1.0`** — remove `Boundary` dependency\n- **`1.0.2`** — fixed `update_meta/3`\n- **`1.0.1`** — fixed different `Access` implementations for `__meta__: ` argument to `use Vela`\n- **`1.0.0`** — cleaned up the code, introduced interfaces, example as docs\n- **`0.17.0`** — type `VelaImpl.slice()` + many docs improvements\n- **`0.16.1`** — `Vela.merge/3`\n- **`0.16.0`** — `Vela.empty?/1`, `Vela.empty!/1` and some performance improvements in `Vela.equal?/2`\n- **`0.15.3`** — allow all metas to reside in `state`\n- **`0.15.2`** — less restrictive `slice/2`, accepting malformed velas\n- **`0.15.0`** — `:mη` keyword parameter in a call to `use Vela` has been renamed to `:__meta__` as _Elixir v1.14_ does not support mixed scripts (meh)\n- **`0.14.0`** — `average/2` to produce a keyword of `{serie, average}` values\n- **`0.13.1`** — `state/1` and `update_state/2` to keep some additional date alongside with `Vela`\n- **`0.12.0`** — `threshold` does not depend on the band anymore\n- **`0.11.0`** — `__meta__` might be now used to overwrite compiled in serie settings\n- **`0.9.5`** — early return the value provided by existing comparison function in `equal?/2`\n- **`0.9.3`** — allow `:atom` and `{GenServer, :on_start}` as type definition\n- **`0.9.0`** — allow a precise type definition of each serie via `type: type()` keyword parameter\n  a series keyword parameter\n- **`0.8.0`** — allow a `corrector/2` callback to allow correction of rejected values as\n  a series keyword parameter\n- **`0.7.2`** — `Vela.put/3`\n- **`0.7.0`** — exact type and behaviour for those using `Vela`\n- **`0.6.3`** — fix `threshold` to use `compare_by/1` for cumbersome values\n- **`0.6.1`** — use `threshold` to prevent adding outliers to series\n- **`0.6.0`** — `Vela.δ/2` / `c:Vela.delta/2` returning a keyword `[{serie, {min, max}}]`\n\n## [Documentation](https://hexdocs.pm/vela)","ref":"readme.html#changelog"},{"type":"extras","title":"Vela Lifetime","doc":"# Vela Lifetime\n\n`Vela` is designed to collect and keep a set of series of different values. Each serie is supposed to keep _validated_, _ordered_, _actual_ values. \n\n- validation is being done upon insertion, using `validator` parameter for this particular serie (_default:_ always allow;) if the validation fails, the value gets discarded immediately\n- obsoleted values are removed by explicit call to `purge/2`\n- sorting is done with `sorter`/`compare_by` pair of parameters; default sorting is none (_LIFO_) and the default comparator is `itself`.\n\nConsider the following example. Let’s declare `Vela` having two series, one with integers, another with dates. We want both to be sorted descending so that we could get the most actual values at any time.\n\n```elixir\ndefmodule Checkers do\n  def good_integer(:integers, int) when is_integer(int), do: true\n  def good_integer(_, _), do: false\n\n  def good_date(:dates, %Date{}), do: true\n  def good_date(_, _), do: false\n\n  def compare_dates(%Date{} = d1, %Date{} = d2),\n    do: Date.compare(d1, d2) == :gt\nend\n\ndefmodule Series do\n  import Checkers\n\n  use Vela,\n    integers: [limit: 3, validator: &good_integer/2, sorter: &>=/2],\n    dates: [limit: 3, validator: &good_date/2, sorter: &compare_dates/2]\nend\n```\n\nNow we can fill the struct with values.\n\n```elixir\niex|1> s = %Series{}\n#⇒ %Series{__errors__: [], dates: [], integers: []}\niex|2> put_in s, [:integers], 42\n#⇒ %Series{__errors__: [], dates: [], integers: '*'}\niex|3> put_in s, [:integers], \"42\"\n#⇒ %Series{__errors__: [integers: \"42\"], dates: [], integers: []}\niex|4> put_in s, [:dates], Date.utc_today\n#⇒ %Series{__errors__: [], dates: [~D[2020-06-30]], integers: []}\niex|5> put_in s, [:dates], \"42\"   \n#⇒ %Series{__errors__: [dates: \"42\"], dates: [], integers: []}\n```\n\nLet’s see how sorting works.\n\n```elixir\niex|6> s\n...|6> |> put_in([:integers], 10)\n...|6> |> put_in([:integers], 20)\n...|6> |> put_in([:integers], 30)\n...|6> |> put_in([:integers], 40)\n...|6> |> put_in([:integers], 0)\n%Series{__errors__: [], dates: [], integers: [40, 30, 20]}\n```\n\nWe instructed the serie to keep at most three values, sorted descending.\n\n---\n\nThe get to the most “relevant” values (heads of series,) one might use `slice/1`. It returns the keyword with all the actual values.\n\n```elixir\niex|7> Series.slice s\n#⇒ [integers: 40]\n```\n\nTo get to the head of each serie, one might use `Access` as shown below.\n\n```elixir\niex|8> get_in s, [:integers]\n#⇒ 40\n```","ref":"lifetime.html"}],"content_type":"text/markdown"}